/**
 * Get css selector of parent node using its children's selector.
 * @param string selector valid css selector
 * @return string of valid css selector if element is found
 * @return undefined if css selector is not found
 */

(function(jQuery) {
	divle_getParentSelector = function(selector){
	  // get parent element
	  var parent = jQuery(jQuery(selector).get(0)).parent();
	
	  try {
	
	    // get css selector of parent element
	    return divle_getElementSelector(parent);
	
	  } catch (e) {
	
	    // error found when finding the parent element
	    return undefined;
	
	  }
	}
	
	/**
	 * Get child nodes' css selector using parent selector.
	 * @param string selector valid css selector
	 * @return array of css selector if child elements is found
	 * @return empty array if child elements is not found
	 */
	divle_getChildrenSelectors = function(selector) {
	  var jQuerynode = jQuery(selector);
	  if(jQuerynode == undefined) return [];
	
	  // iterate over node's children element
	  var selectors = [];
	  jQuerynode.children().each(function(){
	    // get children's valid css selector and collect it inside an array
	    selectors.push(divle_getElementSelector(this));
	  });
	
	  return selectors;
	}
	
	/**
	 * Get css selectors of element which has the same class.
	 * @param string selector valid css selector
	 * @param boolean sorted whether to return the array in sorted list
	 * @return multimensional array of selectors (className: [selectors, ...])
	 * @return empty array the element has no class
	 */
	divle_getRelatedByClassSelectors = function(selector, sorted) {
	  var jQuerythis = jQuery(selector);
	  if(jQuerythis == undefined) return [];
	
	  // get class names
	  var classes = divle_getElementClasses(jQuerythis);
	  var classSelectors = {};
	
	  // iterate over class names
	  jQuery.each(classes, function(){
	    // if no class name exist, define it with an empty array
	    if(classSelectors[this] == undefined) classSelectors[this] = [];
	
	    // merge list of related elements
	    jQuery.merge(classSelectors[this], divle_getRelatedByClass(this, jQuerythis));
	  });
	
	  if(sorted == true) {
	    // sort the list class selectors
	    return divle_sortRelatedClassStack(classSelectors);
	  } else {
	    return classSelectors;
	  }
	}
	
	/**
	 * Organize multi-dimensional array made by getRelatedByClassSelector function.
	 * @param array ClassNames list of class selectors
	 * @return single-dimensional array
	 */
	divle_sortRelatedClassStack = function(classNames) {
	    var classes = [];
	
	    // iterate over class names
	    jQuery.each(classNames, function(name, list){
	        jQuery.each(list, function(i, className){
	            if(jQuery.inArray(className, classes) == -1) {
	                // add class name into the stack if it's not exists inside the stack
	                classes.push(className);
	            }
	        });
	    });
	
	    return classes;
	}
	
	/**
	 * Get css selector with class name as an argument.
	 * @param string className css class
	 * @return array of selectors
	 */
	function divle_getRelatedByClass(className) {
	  var selectors = [];
	
	  // get elements with specified class name
	  jQuery('.' + className).each(function(){
	    // get css selector for each of them
	    selectors.push(divle_getElementSelector(this));
	  });
	
	  return selectors;
	}
	
	/**
	 * Get array of class name from a jQuery element object.
	 * @param object jQueryobj element
	 * @return array
	 */
	divle_getElementClasses = function(jQueryobj) {
	  var tmpClasses = jQueryobj.attr('class');
	
	  // no class name specified
	  if(tmpClasses == undefined) return [];
	
	  var classes = [];
	  // split the class names by spaces
	  tmpClasses = tmpClasses.split(' ');
	
	  // excluded class names
	  var excluded = ['ui-resizable']; 
	
	  jQuery.each(tmpClasses, function(i, className){
	    // if the class name is already exist, ignore it
	    if(jQuery.trim(className) == '' || jQuery.inArray(className, excluded) != -1) return true;
	    classes.push(className);
	
	  });
	
	  return classes;
	}
	
	/**
	 * Get css selector of an element.
	 * @param array single element or list of elements
	 * @return string
	 */
	divle_getElementSelector = function(elements) {
	  if(typeof elements == 'string' ||
	     (typeof elements == 'object' && elements.length == undefined)) {
	
	    // parameter is a single element, define an array with single member
	    var tmpElement = elements;
	    elements = [tmpElement];
	  }
	
	  // get first element
	  element = elements[0];
	
	  // get element's id
	  var elementId = jQuery(element).attr("id");
	
	  if(elementId != undefined) {
	    // element's id is specified
	    var selectorString = '#' + elementId;
	
	    // iterate over list of elements
	    jQuery.each(elements, function(idx, elem){
	      // ignore first element as it's an element with id specified
	      if(idx == 0) return true;
	
	      // get index of child element and append the css selector
	      var childElementIndex = jQuery(elements[idx-1]).children().index(elem);
	      selectorString += ' > :nth-child(' + (childElementIndex + 1) + ')';
	    });
	
	    return selectorString;
	  }
	
	  // check for its classes
	  var elementClasses = jQuery(element).attr("class");
	
	  if(elementClasses != undefined && elements.length == 1) {
	    // element's class is specified
	
	    // get list of class names
	    var elementClassesArray = elementClasses.split(" ");
	
	    // join array of class name with comma
	    var elementClassesString = elementClassesArray.join(", ");
	
	    // use first class name only
	    var firstClassName = elementClassesArray[0];
	
	    // get elements with specified class name
	    jQuery("."+firstClassName).each(function(idx, elem){
	      jQueryelem = jQuery(elem);
	      var w = jQueryelem.outerWidth();
	      var h = jQueryelem.outerHeight();
	      var offset = jQueryelem.offset();
	      //var jQueryredbox = jQuery('<div class="redbox" />');
	      //jQueryelem.after(jQueryredbox);
	      //jQueryredbox.offset(offset).width(w).height(h);
	    });
	
	  }
	
	  if(jQuery(element).is("body")) {
	    // the element is a body (it means it's the topmost element)
	    var selectorString = 'body';
	
	    // iterate over list of elements
	    jQuery.each(elements, function(idx, elem){
	      // ignore current element
	      if(idx == 0) return true;
	
	      // get index of child element and append the css selector
	      var childElementIndex = jQuery(elements[idx-1]).children().index(elem);
	      selectorString += ' > :nth-child(' + (childElementIndex + 1) + ')';
	    });
	
	    return selectorString;
	  }
	
	  // get parent of current element
	  var parentElement = jQuery(element).parent();
	
	  if(parentElement != undefined) {
	    // if current element has parent, run this function recursively
	    return divle_getElementSelector(jQuery.merge(parentElement, elements));
	  }
	}
	
	/**
	 * Stringify array of class name.
	 * @param array classNames list of class names
	 * @return string
	 */
	divle_stringifyClassNames = function(classNames) {
	    var nameStack = [];
	    jQuery.each(classNames, function(){
	        nameStack.push('.' + this);
	    })
	    return nameStack.join(', ');
	}
	
	/** 
	 * check whether argument is a valid css selector
	 * @param string classSelector class selector
	 * @return boolean
	 */
	divle_isValidClassSelector = function(classSelector) {
	    var classes = classSelector.split(',');
	    var isValid = true;
	
	    jQuery.each(classes, function(){
	        var className = jQuery.trim(this);
	        // if the class name has illegal character, the class name is not valid
	        if(className.match(/^\.[\w\-\_]+$/) == null) {
	            isValid = false;
	            return false;
	        }
	    })
	
	    return isValid;
	}
})(jQuery);